<?

/*

 	Webmedia Explorer 3.0 - Multimedia Management System
 	Copyright (C) 2006 Marc Salmurri (marc.salmurri@gmail.com)

 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
 	the Free Software Foundation; either version 2 of the License, or (at
 	your option) any later version.

 	This program is distributed in the hope that it will be useful, but
 	WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 	General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.

*/

set_time_limit ( 0 );

class XMLToArray
{
    var $parser;
	var $parse_ok;
    var $node_stack = array ();

    function XMLToArray ( $xmlstring = "" )
	{
        if ( $xmlstring ) return ( $this->parse ( $xmlstring ) );
        return ( true );
    }

    function parse ( $xmlstring = "" )
	{
        $this->parser = xml_parser_create ();

        xml_set_object ( $this->parser, $this );
        xml_parser_set_option ( $this->parser, XML_OPTION_CASE_FOLDING, false );
        xml_set_element_handler ( $this->parser, "startElement", "endElement" );
        xml_set_character_data_handler ( $this->parser, "characterData" );

        $this->node_stack = array ();
        $this->startElement ( null, "root", array () );

        xml_parse ( $this->parser, $xmlstring );

		if ( xml_get_error_code ( $this->parser )<=0 ) $this->parse_ok = 1;
		else $this->parse_ok = 0;
		
        xml_parser_free ( $this->parser );

        $root_node = array_pop ( $this->node_stack );
		
		return ( $root_node[0] );
    }

    function startElement ( $parser, $name, $attrs )
	{
        $node = array ();
        $node["name"] = $name;

        foreach ( $attrs as $key => $value )
		{
			$node["attrib"][$key] = $value;
		}

        array_push ( $this->node_stack, $node );
    }

    function endElement ( $parser, $name )
	{
        $node = array_pop ( $this->node_stack );

		if ( eregi ( "[a-zA-Z]+", $data ) )	$node["content"] = $node["content"];

        $lastnode = count ( $this->node_stack ) - 1;

		array_push ( $this->node_stack[$lastnode], $node );
    }

    function characterData ( $parser, $data )
	{
        $lastnode = count ( $this->node_stack ) - 1;

		$i = count ( $this->node_stack[$lastnode] ) - 1;

		$this->node_stack[$lastnode][$i]["content"] = $data;
    }
}

define ( xelements, 1 );
define ( xcontent, 2 );

function utf8dec ( $buffer )
{
	$buffer = preg_replace ( "/([\xC2\xC3])([\x80-\xBF])/e", "chr(ord('\\1')<<6&0xC0|ord('\\2')&0x3F)", $buffer ); 

	$buffer = str_replace ( "€", "", $buffer );

	$buffer = str_replace ( "…", "...", $buffer );

	$buffer = str_replace ( "–", "-", $buffer );

	$buffer = str_replace ( "’", "'", $buffer );

	$buffer = str_replace ( "“", "\"", $buffer );

	$buffer = str_replace ( "", "\"", $buffer );

	return $buffer;
}

class xdoc
{
	var $xml;
	var $data;
	var $root;
	var $i;
	var $template;
	var $parse_ok;

	var $_this;

	function xdoc ( $data, $encoding = "" )
    {
		if ( is_array ( $data ) ) $this->data = $data;

		else
		{
			$xml_data = new XMLToArray ();
			
			if ( $encoding == "utf8" ) $data = utf8dec ( $data );

			$this->data = $xml_data->parse ( $data );
			
			$this->parse_ok = $xml_data->parse_ok;
			
			$this->root = $this->data;
		}
    }

	function render ( $template = "", $type = 0 )
    {
		global $path_template;
		global $lang;

		if ( empty ( $template ) ) $template = $this->template;

		else $this->template = $template;

		for ( $this->i = 0; $this->i < count ( $this->data ); $this->i++ )
		{
			if ( ( ( $type == 0 ) && ( isset ( $this->data[$this->i]["name"] ) || isset ( $this->data[$this->i]["content"] ) ) )
			|| ( ( $type == 1 ) && ( isset ( $this->data[$this->i]["name"] ) ) )
			|| ( ( $type == 2 ) && ( isset ( $this->data[$this->i]["content"] ) ) ) )
			{
				$meta = $this->meta;
				$_this = $this->_this;
				$parent = $this->data;
				$root = $this->root;

				if ( isset ( $this->data[$this->i]["name"] ) ) $element_name = $this->data[$this->i]["name"];

				else if ( isset ( $this->data[$this->i]["content"] ) ) $element_name = "element";

				$$element_name = new xdoc ( $this->data[$this->i] );
				$$element_name->template = $template;

                $$element_name->meta = $this->meta;
                $$element_name->_this = $this->_this;
                $$element_name->parent = $this->data;
				$$element_name->root = $this->root;

				include ( $path_template.$this->template );
			}
		}
    }
	
	function render_buffer ( $template = "" )
	{
		ob_start ();
        
		$this->render ( $template );
		
		$buffer = ob_get_clean ();
		
		return $buffer;
	}

    function extract_content ( $data, $limit = 225 )
    {
        $buffer = "";

        $not_long_enuf = true;

        for ( $i = 0; $not_long_enuf && $i < count ( $data ); $i++ )
        {
			if ( strlen ( $buffer.$data[$i]["content"] ) < $limit )
            {
                $buffer .= $data[$i]["content"];
            }
            else
			{
				$buffer .= subtxt ( $data[$i]["content"], $limit - strlen ( $buffer ) );
				
				$not_long_enuf = false;
			}
        }

		return str_replace ( "\n", " ", $buffer );
    }
}

?>
